#include <os.h>
#include "console.h"
#include "screen.h"
#include "charmap.h"
#define MEM1	0x900B0000
#define MEM2	0x900B000C

int getMem(int addr)
{	return *(volatile unsigned*) addr;
}

void setMem(int addr, int val)
{	*(volatile unsigned*) addr = val;
}

void dispcpu(int b1, int b2, int r1, int r2)
{	int base = 27;
	r1*=2;
	r2++;
	if(!b1) base = 300 - 6*b2;
	disp  ("    Base = ",0); dispi(base      ,0);
	setConsoleCol(15);
	disp  ("MHz   Base / CPU = ",0); dispi(r1        ,0); displn("",0);
	disp  ("    CPU  = ",0); if(r1) dispi(base/r1   ,0); else disp("?",0);
	setConsoleCol(15);
	disp  ("MHz   CPU  / AHB = ",0); dispi(r2        ,0); displn("",0);
	disp  ("    AHB  = ",0); if(r1) dispi(base/r1/r2,0); else disp("?",0);
	setConsoleCol(15);
	disp  ("MHz   Base / AHB = ",0); dispi(r1*r2     ,0); displn("",0);
}

int makeConfig(int b1, int b2, int r1, int r2)
{	return (b2<<16)|(r2<<12)|(b1<<8)|(r1<<1);
}

int main(int argc, char* argv[])
{	int orig_config = getMem(MEM1);
	int b1_orig = (orig_config & 0b000000000000100000000)>>8;
	int b2_orig = (orig_config & 0b111110000000000000000)>>16;
	int r1_orig = (orig_config & 0b000000000000011111110)>>1;
	int r2_orig = (orig_config & 0b000000111000000000000)>>12;
	int b1_curr = b1_orig;
	int b2_curr = b2_orig;
	if(b1_curr) b2_curr=0b11111;
	int r1_curr = r1_orig;
	int r2_curr = r2_orig;
	int b1_last = b1_curr;
	int b2_last = b2_curr;
	int r1_last = r1_curr;
	int r2_last = r2_curr;
	int c;
	int m=0;
	int last_config=orig_config;
	int curr_config;
	while(!isKeyPressed(KEY_NSPIRE_ESC))
	{	c=0;
		clrScr();
		resetConsole();
		displn("Nover 1.1  ( http://www.ti-bank.fr/913 )",0);
		displn(LSEPARATOR,0);
		displn("",0);
		curr_config=makeConfig(b1_curr,b2_curr,r1_curr,r2_curr);
		disp("Current raw config: ",0);dispi(curr_config,0);
		if(curr_config!=last_config)
			displn("  (not set)",0);
		else	displn("  (  set  )",0);
		displn(LSEPARATOR,0);
		dispcpu(b1_curr,b2_curr, r1_curr, r2_curr);
		displn(LSEPARATOR,0);
		displn("   [1] Base  [2] Base/CPU  [3] CPU/AHB",0);
		if(m)
		{	if(m==1)      disp("Base frequency:  ",0);
			else if(m==2) disp("Base/CPU ratio:  ",0);
			else if(m==3) disp("CPU/AHB ratio :  ",0);
			displn("[+]Increase [-]Decrease",0);
		}
		drwStr(0,(MAX_LGN-9)*CHAR_HEIGHT,"[  0  ] Cancel all   modifications",0);
		drwStr(0,(MAX_LGN-8)*CHAR_HEIGHT,"[ (-) ] Cancel unset modifications",0);
		drwStr(0,(MAX_LGN-7)*CHAR_HEIGHT,"[  9  ] Load fastest  config OS  2.1",0);
		drwStr(0,(MAX_LGN-6)*CHAR_HEIGHT,"[  6  ] Load official config OS  2.1",0);
		drwStr(0,(MAX_LGN-5)*CHAR_HEIGHT,"[  x  ] Load fastest  config OS  2.0",0);
		drwStr(0,(MAX_LGN-4)*CHAR_HEIGHT,"[    ] Load official config OS  2.0",0);
		drwStr(0,(MAX_LGN-2)*CHAR_HEIGHT,"[enter] Sets current config",0);
		drwStr(0,(MAX_LGN-1)*CHAR_HEIGHT,"[ esc ] Exit program",0);
		while(!isKeyPressed(KEY_NSPIRE_ESC) && !c)
		{	if(isKeyPressed(KEY_NSPIRE_1) && m!=1)
			{	m=1; c=1; }
			else if((isKeyPressed(KEY_NSPIRE_2) || isKeyPressed(KEY_NSPIRE_B)) && m!=2)
			{	m=2; c=1; }
			else if(isKeyPressed(KEY_NSPIRE_3) && m!=3)
			{	m=3; c=1; }
			if(m==1)
			{	if((isKeyPressed(KEY_NSPIRE_PLUS) || isKeyPressed(KEY_NSPIRE_C)) && b2_curr>0)
				{	if(b1_curr)
						b1_curr=0;
					else	b2_curr--;
					c=1;
				}
				if((isKeyPressed(KEY_NSPIRE_MINUS) || isKeyPressed(KEY_NSPIRE_G)) && !b1_curr)
				{	if(b2_curr<0b11111)
						b2_curr++;
					else
						b1_curr=1;
					c=1;
				}
			}
			else if(m==2)
			{	if((isKeyPressed(KEY_NSPIRE_PLUS) || isKeyPressed(KEY_NSPIRE_C)) && r1_curr<0b1111111)
				{	r1_curr++;
					c=1;
				}
				if((isKeyPressed(KEY_NSPIRE_MINUS) || isKeyPressed(KEY_NSPIRE_G)) && r1_curr>0)
				{	r1_curr--;
					c=1;
				}
			}
			else if(m==3)
			{	if((isKeyPressed(KEY_NSPIRE_PLUS) || isKeyPressed(KEY_NSPIRE_C))&& r2_curr<0b111)
				{	r2_curr++;
					c=1;
				}
				if((isKeyPressed(KEY_NSPIRE_MINUS) || isKeyPressed(KEY_NSPIRE_G))&& r2_curr>0)
				{	r2_curr--;
					c=1;
				}
			}
			if(isKeyPressed(KEY_NSPIRE_0))
			{	r1_curr=r1_orig;
				r2_curr=r2_orig;
				b1_curr=b1_orig;
				b2_curr=b2_orig;
				c=1;
			}
			if(isKeyPressed(KEY_NSPIRE_NEGATIVE))
			{	r1_curr=r1_last;
				r2_curr=r2_last;
				b1_curr=b1_last;
				b2_curr=b2_last;
				c=1;
			}
			if(isKeyPressed(KEY_NSPIRE_DIVIDE) || isKeyPressed(KEY_NSPIRE_L))
			{	r1_curr=1;
				r2_curr=5;
				b1_curr=0;
				b2_curr=20;
				c=1;
			}
			if(isKeyPressed(KEY_NSPIRE_MULTIPLY) || isKeyPressed(KEY_NSPIRE_H))
			{	r1_curr=1;
				r2_curr=1;
				b1_curr=0;
				b2_curr=20;
				c=1;
			}
			if(isKeyPressed(KEY_NSPIRE_9))
			{	r1_curr=1;
				r2_curr=1;
				b1_curr=0;
				b2_curr=0;
				c=1;
			}
			if(isKeyPressed(KEY_NSPIRE_6))
			{	r1_curr=1;
				r2_curr=1;
				b1_curr=0;
				b2_curr=10;
				c=1;
			}
			if(isKeyPressed(KEY_NSPIRE_ENTER))
			{	last_config=makeConfig(b1_curr,b2_curr,r1_curr,r2_curr);
				b1_last=b1_curr;
				b2_last=b2_curr;
				r1_last=r1_curr;
				r2_last=r2_curr;
				setMem(MEM1,last_config);
				setMem(MEM2,4);
				c=1;
			}
			if(c) wait(200);
		}
	}
	return 0;
}